/*
 * Copyright 2001-2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.easymock.internal.classinfoprovider;

/**
 * Tells where a mock should be created.
 */
public interface ClassInfoProvider {

    /**
     * Returns the package where the mock should be created.
     *
     * @param toMock the class to mock
     * @return the package name
     */
    String classPackage(Class<?> toMock);

    /**
     * Returns the class loader where the mock should be created.
     *
     * @param toMock the class to mock
     * @return the class loader where the mock should be created
     * @param <T> type of the class to mock
     */
    <T> ClassLoader classLoader(Class<T> toMock);

}
