/*
 * Copyright 2018-2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.easymock.java8;

/**
 * @author Henri Tremblay
 */
public class Methods {

    public static class NoDefaultOverride implements IMethods {
        @Override
        public int normalInterfaceMethod() {
            return defaultInterfaceMethod();
        }
    }

    public static class DefaultOverride implements IMethods {
        @Override
        public int normalInterfaceMethod() {
            return defaultInterfaceMethod();
        }

        @Override
        public int defaultInterfaceMethod() {
            return 2;
        }
    }

    public static class A {
        public int defaultInterfaceMethod() {
            return 3;
        }
    }

    public static class B extends A implements IMethods {
        @Override
        public int normalInterfaceMethod() {
            return defaultInterfaceMethod();
        }
    }
}
